@@ -3,7 +3,7 @@
  *
  * \brief Blowfish block cipher
  *
- *  Copyright (C) 2012-2013, Brainspark B.V.
+ *  Copyright (C) 2012-2014, Brainspark B.V.
  *
  *  This file is part of PolarSSL (http://www.polarssl.org)
  *  Lead Maintainer: Paul Bakker <polarssl_maintainer at polarssl.org>
@@ -27,7 +27,11 @@
 #ifndef POLARSSL_BLOWFISH_H
 #define POLARSSL_BLOWFISH_H
 
+#if !defined(POLARSSL_CONFIG_FILE)
 #include "config.h"
+#else
+#include POLARSSL_CONFIG_FILE
+#endif
 
 #include <string.h>
 
@@ -42,7 +46,7 @@ typedef UINT32 uint32_t;
 #define BLOWFISH_DECRYPT     0
 #define BLOWFISH_MAX_KEY     448
 #define BLOWFISH_MIN_KEY     32
-#define BLOWFISH_ROUNDS      16         /* when increasing this value, make sure to extend the initialisation vectors */
+#define BLOWFISH_ROUNDS      16         /**< Rounds to use. When increasing this value, make sure to extend the initialisation vectors */
 #define BLOWFISH_BLOCKSIZE   8          /* Blowfish uses 64 bit blocks */
 
 #define POLARSSL_ERR_BLOWFISH_INVALID_KEY_LENGTH                -0x0016  /**< Invalid key length. */
@@ -75,7 +79,8 @@ blowfish_context;
  *
  * \return         0 if successful, or POLARSSL_ERR_BLOWFISH_INVALID_KEY_LENGTH
  */
-int blowfish_setkey( blowfish_context *ctx, const unsigned char *key, unsigned int keysize );
+int blowfish_setkey( blowfish_context *ctx, const unsigned char *key,
+                     unsigned int keysize );
 
 /**
  * \brief          Blowfish-ECB block encryption/decryption
@@ -105,7 +110,8 @@ int blowfish_crypt_ecb( blowfish_context *ctx,
  * \param input    buffer holding the input data
  * \param output   buffer holding the output data
  *
- * \return         0 if successful, or POLARSSL_ERR_BLOWFISH_INVALID_INPUT_LENGTH
+ * \return         0 if successful, or
+ *                 POLARSSL_ERR_BLOWFISH_INVALID_INPUT_LENGTH
  */
 int blowfish_crypt_cbc( blowfish_context *ctx,
                         int mode,
