@@ -5,7 +5,7 @@
  *
  * \author Adriaan de Jong <dejong@fox-it.com>
  *
- *  Copyright (C) 2006-2013, Brainspark B.V.
+ *  Copyright (C) 2006-2014, Brainspark B.V.
  *
  *  This file is part of PolarSSL (http://www.polarssl.org)
  *  Lead Maintainer: Paul Bakker <polarssl_maintainer at polarssl.org>
@@ -27,7 +27,11 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#if !defined(POLARSSL_CONFIG_FILE)
 #include "polarssl/config.h"
+#else
+#include POLARSSL_CONFIG_FILE
+#endif
 
 #if defined(POLARSSL_MD_C)
 
@@ -61,8 +65,8 @@
 #include "polarssl/sha512.h"
 #endif
 
-#if defined(POLARSSL_MEMORY_C)
-#include "polarssl/memory.h"
+#if defined(POLARSSL_PLATFORM_C)
+#include "polarssl/platform.h"
 #else
 #define polarssl_malloc     malloc
 #define polarssl_free       free
@@ -77,7 +81,8 @@ static void md2_starts_wrap( void *ctx )
     md2_starts( (md2_context *) ctx );
 }
 
-static void md2_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void md2_update_wrap( void *ctx, const unsigned char *input,
+                             size_t ilen )
 {
     md2_update( (md2_context *) ctx, input, ilen );
 }
@@ -98,12 +103,14 @@ static int md2_file_wrap( const char *path, unsigned char *output )
 #endif
 }
 
-static void md2_hmac_starts_wrap( void *ctx, const unsigned char *key, size_t keylen )
+static void md2_hmac_starts_wrap( void *ctx, const unsigned char *key,
+                                  size_t keylen )
 {
     md2_hmac_starts( (md2_context *) ctx, key, keylen );
 }
 
-static void md2_hmac_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void md2_hmac_update_wrap( void *ctx, const unsigned char *input,
+                                  size_t ilen )
 {
     md2_hmac_update( (md2_context *) ctx, input, ilen );
 }
@@ -154,7 +161,7 @@ const md_info_t md2_info = {
     md2_process_wrap,
 };
 
-#endif
+#endif /* POLARSSL_MD2_C */
 
 #if defined(POLARSSL_MD4_C)
 
@@ -163,7 +170,8 @@ static void md4_starts_wrap( void *ctx )
     md4_starts( (md4_context *) ctx );
 }
 
-static void md4_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void md4_update_wrap( void *ctx, const unsigned char *input,
+                             size_t ilen )
 {
     md4_update( (md4_context *) ctx, input, ilen );
 }
@@ -184,12 +192,14 @@ static int md4_file_wrap( const char *path, unsigned char *output )
 #endif
 }
 
-static void md4_hmac_starts_wrap( void *ctx, const unsigned char *key, size_t keylen )
+static void md4_hmac_starts_wrap( void *ctx, const unsigned char *key,
+                                  size_t keylen )
 {
     md4_hmac_starts( (md4_context *) ctx, key, keylen );
 }
 
-static void md4_hmac_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void md4_hmac_update_wrap( void *ctx, const unsigned char *input,
+                                  size_t ilen )
 {
     md4_hmac_update( (md4_context *) ctx, input, ilen );
 }
@@ -238,7 +248,7 @@ const md_info_t md4_info = {
     md4_process_wrap,
 };
 
-#endif
+#endif /* POLARSSL_MD4_C */
 
 #if defined(POLARSSL_MD5_C)
 
@@ -247,7 +257,8 @@ static void md5_starts_wrap( void *ctx )
     md5_starts( (md5_context *) ctx );
 }
 
-static void md5_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void md5_update_wrap( void *ctx, const unsigned char *input,
+                             size_t ilen )
 {
     md5_update( (md5_context *) ctx, input, ilen );
 }
@@ -268,12 +279,14 @@ static int md5_file_wrap( const char *path, unsigned char *output )
 #endif
 }
 
-static void md5_hmac_starts_wrap( void *ctx, const unsigned char *key, size_t keylen )
+static void md5_hmac_starts_wrap( void *ctx, const unsigned char *key,
+                                  size_t keylen )
 {
     md5_hmac_starts( (md5_context *) ctx, key, keylen );
 }
 
-static void md5_hmac_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void md5_hmac_update_wrap( void *ctx, const unsigned char *input,
+                                  size_t ilen )
 {
     md5_hmac_update( (md5_context *) ctx, input, ilen );
 }
@@ -322,7 +335,7 @@ const md_info_t md5_info = {
     md5_process_wrap,
 };
 
-#endif
+#endif /* POLARSSL_MD5_C */
 
 #if defined(POLARSSL_RIPEMD160_C)
 
@@ -331,7 +344,8 @@ static void ripemd160_starts_wrap( void *ctx )
     ripemd160_starts( (ripemd160_context *) ctx );
 }
 
-static void ripemd160_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void ripemd160_update_wrap( void *ctx, const unsigned char *input,
+                                   size_t ilen )
 {
     ripemd160_update( (ripemd160_context *) ctx, input, ilen );
 }
@@ -352,12 +366,14 @@ static int ripemd160_file_wrap( const char *path, unsigned char *output )
 #endif
 }
 
-static void ripemd160_hmac_starts_wrap( void *ctx, const unsigned char *key, size_t keylen )
+static void ripemd160_hmac_starts_wrap( void *ctx, const unsigned char *key,
+                                        size_t keylen )
 {
     ripemd160_hmac_starts( (ripemd160_context *) ctx, key, keylen );
 }
 
-static void ripemd160_hmac_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void ripemd160_hmac_update_wrap( void *ctx, const unsigned char *input,
+                                        size_t ilen )
 {
     ripemd160_hmac_update( (ripemd160_context *) ctx, input, ilen );
 }
@@ -406,7 +422,7 @@ const md_info_t ripemd160_info = {
     ripemd160_process_wrap,
 };
 
-#endif
+#endif /* POLARSSL_RIPEMD160_C */
 
 #if defined(POLARSSL_SHA1_C)
 
@@ -415,7 +431,8 @@ static void sha1_starts_wrap( void *ctx )
     sha1_starts( (sha1_context *) ctx );
 }
 
-static void sha1_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void sha1_update_wrap( void *ctx, const unsigned char *input,
+                              size_t ilen )
 {
     sha1_update( (sha1_context *) ctx, input, ilen );
 }
@@ -436,12 +453,14 @@ static int sha1_file_wrap( const char *path, unsigned char *output )
 #endif
 }
 
-static void sha1_hmac_starts_wrap( void *ctx, const unsigned char *key, size_t keylen )
+static void sha1_hmac_starts_wrap( void *ctx, const unsigned char *key,
+                                   size_t keylen )
 {
     sha1_hmac_starts( (sha1_context *) ctx, key, keylen );
 }
 
-static void sha1_hmac_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void sha1_hmac_update_wrap( void *ctx, const unsigned char *input,
+                                   size_t ilen )
 {
     sha1_hmac_update( (sha1_context *) ctx, input, ilen );
 }
@@ -490,7 +509,7 @@ const md_info_t sha1_info = {
     sha1_process_wrap,
 };
 
-#endif
+#endif /* POLARSSL_SHA1_C */
 
 /*
  * Wrappers for generic message digests
@@ -502,7 +521,8 @@ static void sha224_starts_wrap( void *ctx )
     sha256_starts( (sha256_context *) ctx, 1 );
 }
 
-static void sha224_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void sha224_update_wrap( void *ctx, const unsigned char *input,
+                                size_t ilen )
 {
     sha256_update( (sha256_context *) ctx, input, ilen );
 }
@@ -529,12 +549,14 @@ static int sha224_file_wrap( const char *path, unsigned char *output )
 #endif
 }
 
-static void sha224_hmac_starts_wrap( void *ctx, const unsigned char *key, size_t keylen )
+static void sha224_hmac_starts_wrap( void *ctx, const unsigned char *key,
+                                     size_t keylen )
 {
     sha256_hmac_starts( (sha256_context *) ctx, key, keylen, 1 );
 }
 
-static void sha224_hmac_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void sha224_hmac_update_wrap( void *ctx, const unsigned char *input,
+                                     size_t ilen )
 {
     sha256_hmac_update( (sha256_context *) ctx, input, ilen );
 }
@@ -595,7 +617,8 @@ static void sha256_starts_wrap( void *ctx )
     sha256_starts( (sha256_context *) ctx, 0 );
 }
 
-static void sha256_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void sha256_update_wrap( void *ctx, const unsigned char *input,
+                                size_t ilen )
 {
     sha256_update( (sha256_context *) ctx, input, ilen );
 }
@@ -622,12 +645,14 @@ static int sha256_file_wrap( const char *path, unsigned char *output )
 #endif
 }
 
-static void sha256_hmac_starts_wrap( void *ctx, const unsigned char *key, size_t keylen )
+static void sha256_hmac_starts_wrap( void *ctx, const unsigned char *key,
+                                     size_t keylen )
 {
     sha256_hmac_starts( (sha256_context *) ctx, key, keylen, 0 );
 }
 
-static void sha256_hmac_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void sha256_hmac_update_wrap( void *ctx, const unsigned char *input,
+                                     size_t ilen )
 {
     sha256_hmac_update( (sha256_context *) ctx, input, ilen );
 }
@@ -683,7 +708,7 @@ const md_info_t sha256_info = {
     sha256_process_wrap,
 };
 
-#endif
+#endif /* POLARSSL_SHA256_C */
 
 #if defined(POLARSSL_SHA512_C)
 
@@ -692,7 +717,8 @@ static void sha384_starts_wrap( void *ctx )
     sha512_starts( (sha512_context *) ctx, 1 );
 }
 
-static void sha384_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void sha384_update_wrap( void *ctx, const unsigned char *input,
+                                size_t ilen )
 {
     sha512_update( (sha512_context *) ctx, input, ilen );
 }
@@ -719,12 +745,14 @@ static int sha384_file_wrap( const char *path, unsigned char *output )
 #endif
 }
 
-static void sha384_hmac_starts_wrap( void *ctx, const unsigned char *key, size_t keylen )
+static void sha384_hmac_starts_wrap( void *ctx, const unsigned char *key,
+                                     size_t keylen )
 {
     sha512_hmac_starts( (sha512_context *) ctx, key, keylen, 1 );
 }
 
-static void sha384_hmac_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void sha384_hmac_update_wrap( void *ctx, const unsigned char *input,
+                                     size_t ilen )
 {
     sha512_hmac_update( (sha512_context *) ctx, input, ilen );
 }
@@ -785,7 +813,8 @@ static void sha512_starts_wrap( void *ctx )
     sha512_starts( (sha512_context *) ctx, 0 );
 }
 
-static void sha512_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void sha512_update_wrap( void *ctx, const unsigned char *input,
+                                size_t ilen )
 {
     sha512_update( (sha512_context *) ctx, input, ilen );
 }
@@ -812,12 +841,14 @@ static int sha512_file_wrap( const char *path, unsigned char *output )
 #endif
 }
 
-static void sha512_hmac_starts_wrap( void *ctx, const unsigned char *key, size_t keylen )
+static void sha512_hmac_starts_wrap( void *ctx, const unsigned char *key,
+                                     size_t keylen )
 {
     sha512_hmac_starts( (sha512_context *) ctx, key, keylen, 0 );
 }
 
-static void sha512_hmac_update_wrap( void *ctx, const unsigned char *input, size_t ilen )
+static void sha512_hmac_update_wrap( void *ctx, const unsigned char *input,
+                                     size_t ilen )
 {
     sha512_hmac_update( (sha512_context *) ctx, input, ilen );
 }
@@ -873,6 +904,6 @@ const md_info_t sha512_info = {
     sha512_process_wrap,
 };
 
-#endif
+#endif /* POLARSSL_SHA512_C */
 
-#endif
+#endif /* POLARSSL_MD_C */
