@@ -23,7 +23,11 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#if !defined(POLARSSL_CONFIG_FILE)
 #include "polarssl/config.h"
+#else
+#include POLARSSL_CONFIG_FILE
+#endif
 
 #include <string.h>
 #include <stdio.h>
@@ -150,6 +154,8 @@ static int generic_check( const md_info_t *md_info, char *filename )
                 "not match\n", nb_err2, nb_tot2 );
     }
 
+    fclose( f );
+
     return( nb_err1 != 0 || nb_err2 != 0 );
 }
 
