@@ -23,7 +23,11 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#if !defined(POLARSSL_CONFIG_FILE)
 #include "polarssl/config.h"
+#else
+#include POLARSSL_CONFIG_FILE
+#endif
 
 #include <string.h>
 #include <stdio.h>
@@ -273,7 +277,9 @@ int main( int argc, char *argv[] )
 
 exit:
 
-    net_close( client_fd );
+    if( client_fd != -1 )
+        net_close( client_fd );
+
     rsa_free( &rsa );
     dhm_free( &dhm );
     entropy_free( &entropy );
