@@ -23,7 +23,11 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#if !defined(POLARSSL_CONFIG_FILE)
 #include "polarssl/config.h"
+#else
+#include POLARSSL_CONFIG_FILE
+#endif
 
 #include <string.h>
 #include <stdio.h>
@@ -58,7 +62,7 @@ int main( int argc, char *argv[] )
 int main( int argc, char *argv[] )
 {
     FILE *f;
-    int ret;
+    int ret = 1;
     pk_context pk;
     entropy_context entropy;
     ctr_drbg_context ctr_drbg;
@@ -68,7 +72,8 @@ int main( int argc, char *argv[] )
     const char *pers = "rsa_sign_pss";
     size_t olen = 0;
 
-    ret = 1;
+    entropy_init( &entropy );
+    pk_init( &pk );
 
     if( argc != 3 )
     {
@@ -84,7 +89,6 @@ int main( int argc, char *argv[] )
     printf( "\n  . Seeding the random number generator..." );
     fflush( stdout );
 
-    entropy_init( &entropy );
     if( ( ret = ctr_drbg_init( &ctr_drbg, entropy_func, &entropy,
                                (const unsigned char *) pers,
                                strlen( pers ) ) ) != 0 )
@@ -96,12 +100,11 @@ int main( int argc, char *argv[] )
     printf( "\n  . Reading private key from '%s'", argv[1] );
     fflush( stdout );
 
-    pk_init( &pk );
-
     if( ( ret = pk_parse_keyfile( &pk, argv[1], "" ) ) != 0 )
     {
         ret = 1;
-        printf( " failed\n  ! Could not open '%s'\n", argv[1] );
+        printf( " failed\n  ! Could not read key from '%s'\n", argv[1] );
+        printf( "  ! pk_parse_public_keyfile returned %d\n\n", ret );
         goto exit;
     }
 
@@ -112,6 +115,8 @@ int main( int argc, char *argv[] )
         goto exit;
     }
 
+    rsa_set_padding( pk_rsa( pk ), RSA_PKCS_V21, POLARSSL_MD_SHA1 );
+
     /*
      * Compute the SHA-1 hash of the input file,
      * then calculate the RSA signature of the hash.
