@@ -23,7 +23,11 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#if !defined(POLARSSL_CONFIG_FILE)
 #include "polarssl/config.h"
+#else
+#include POLARSSL_CONFIG_FILE
+#endif
 
 #include <string.h>
 #include <stdlib.h>
@@ -45,18 +49,20 @@
 #include "polarssl/gcm.h"
 #include "polarssl/havege.h"
 #include "polarssl/ctr_drbg.h"
+#include "polarssl/hmac_drbg.h"
 #include "polarssl/rsa.h"
 #include "polarssl/dhm.h"
 #include "polarssl/ecdsa.h"
 #include "polarssl/ecdh.h"
+#include "polarssl/error.h"
 
 #if defined _MSC_VER && !defined snprintf
 #define snprintf _snprintf
 #endif
 
 #define BUFSIZE         1024
-#define HEADER_FORMAT   "  %-18s :  "
-#define TITLE_LEN       19
+#define HEADER_FORMAT   "  %-24s :  "
+#define TITLE_LEN       25
 
 #if !defined(POLARSSL_TIMING_C)
 int main( int argc, char *argv[] )
@@ -131,7 +137,10 @@ do {                                                                    \
     }                                                                   \
                                                                         \
     if( ret != 0 )                                                      \
-        printf( "FAILED\n" );                                           \
+    {                                                                   \
+        polarssl_strerror( ret, ( char * )tmp, sizeof( tmp ) );         \
+        printf( "FAILED: %s\n", tmp );                                  \
+    }                                                                   \
     else                                                                \
         printf( "%9lu " TYPE "/s\n", i / 3 );                           \
 } while( 0 )
@@ -141,14 +150,14 @@ unsigned char buf[BUFSIZE];
 typedef struct {
     char md4, md5, ripemd160, sha1, sha256, sha512,
          arc4, des3, des, aes_cbc, aes_gcm, camellia, blowfish,
-         havege, ctr_drbg,
+         havege, ctr_drbg, hmac_drbg,
          rsa, dhm, ecdsa, ecdh;
 } todo_list;
 
 #define OPTIONS                                                         \
     "md4, md5, ripemd160, sha1, sha256, sha512,\n"                      \
     "arc4, des3, des, aes_cbc, aes_gcm, camellia, blowfish,\n"          \
-    "havege, ctr_drbg,\n"                                               \
+    "havege, ctr_drbg, hmac_drbg\n"                                     \
     "rsa, dhm, ecdsa, ecdh.\n"
 
 int main( int argc, char *argv[] )
@@ -196,6 +205,8 @@ int main( int argc, char *argv[] )
                 todo.havege = 1;
             else if( strcmp( argv[i], "ctr_drbg" ) == 0 )
                 todo.ctr_drbg = 1;
+            else if( strcmp( argv[i], "hmac_drbg" ) == 0 )
+                todo.hmac_drbg = 1;
             else if( strcmp( argv[i], "rsa" ) == 0 )
                 todo.rsa = 1;
             else if( strcmp( argv[i], "dhm" ) == 0 )
@@ -215,6 +226,7 @@ int main( int argc, char *argv[] )
     printf( "\n" );
 
     memset( buf, 0xAA, sizeof( buf ) );
+    memset( tmp, 0xBB, sizeof( tmp ) );
 
 #if defined(POLARSSL_MD4_C)
     if( todo.md4 )
@@ -380,6 +392,56 @@ int main( int argc, char *argv[] )
     }
 #endif
 
+#if defined(POLARSSL_HMAC_DRBG_C)
+    if( todo.hmac_drbg )
+    {
+        hmac_drbg_context hmac_drbg;
+        const md_info_t *md_info;
+
+#if defined(POLARSSL_SHA1_C)
+        if( ( md_info = md_info_from_type( POLARSSL_MD_SHA1 ) ) == NULL )
+            exit(1);
+
+        if( hmac_drbg_init( &hmac_drbg, md_info, myrand, NULL, NULL, 0 ) != 0 )
+            exit(1);
+        TIME_AND_TSC( "HMAC_DRBG SHA-1 (NOPR)",
+                if( hmac_drbg_random( &hmac_drbg, buf, BUFSIZE ) != 0 )
+                exit(1) );
+        hmac_drbg_free( &hmac_drbg );
+
+        if( hmac_drbg_init( &hmac_drbg, md_info, myrand, NULL, NULL, 0 ) != 0 )
+            exit(1);
+        hmac_drbg_set_prediction_resistance( &hmac_drbg,
+                                             POLARSSL_HMAC_DRBG_PR_ON );
+        TIME_AND_TSC( "HMAC_DRBG SHA-1 (PR)",
+                if( hmac_drbg_random( &hmac_drbg, buf, BUFSIZE ) != 0 )
+                exit(1) );
+        hmac_drbg_free( &hmac_drbg );
+#endif
+
+#if defined(POLARSSL_SHA256_C)
+        if( ( md_info = md_info_from_type( POLARSSL_MD_SHA256 ) ) == NULL )
+            exit(1);
+
+        if( hmac_drbg_init( &hmac_drbg, md_info, myrand, NULL, NULL, 0 ) != 0 )
+            exit(1);
+        TIME_AND_TSC( "HMAC_DRBG SHA-256 (NOPR)",
+                if( hmac_drbg_random( &hmac_drbg, buf, BUFSIZE ) != 0 )
+                exit(1) );
+        hmac_drbg_free( &hmac_drbg );
+
+        if( hmac_drbg_init( &hmac_drbg, md_info, myrand, NULL, NULL, 0 ) != 0 )
+            exit(1);
+        hmac_drbg_set_prediction_resistance( &hmac_drbg,
+                                             POLARSSL_HMAC_DRBG_PR_ON );
+        TIME_AND_TSC( "HMAC_DRBG SHA-256 (PR)",
+                if( hmac_drbg_random( &hmac_drbg, buf, BUFSIZE ) != 0 )
+                exit(1) );
+        hmac_drbg_free( &hmac_drbg );
+#endif
+    }
+#endif
+
 #if defined(POLARSSL_RSA_C) && defined(POLARSSL_GENPRIME)
     if( todo.rsa )
     {
@@ -426,11 +488,16 @@ int main( int argc, char *argv[] )
         {
             memset( &dhm, 0, sizeof( dhm_context ) );
 
-            mpi_read_string( &dhm.P, 16, dhm_P[i] );
-            mpi_read_string( &dhm.G, 16, dhm_G[i] );
+            if( mpi_read_string( &dhm.P, 16, dhm_P[i] ) != 0 ||
+                mpi_read_string( &dhm.G, 16, dhm_G[i] ) != 0 )
+            {
+                exit( 1 );
+            }
+
             dhm.len = mpi_size( &dhm.P );
             dhm_make_public( &dhm, (int) dhm.len, buf, dhm.len, myrand, NULL );
-            mpi_copy( &dhm.GY, &dhm.GX );
+            if( mpi_copy( &dhm.GY, &dhm.GX ) != 0 )
+                exit( 1 );
 
             snprintf( title, sizeof( title ), "DHE-%d", dhm_sizes[i] );
             TIME_PUBLIC( title, "handshake",
